/*
 *   Copyright (©) 2009 | 16 January 2009 | EPFL (Ecole Polytechnique fédérale de Lausanne)
 *
 *   TuringSim is free software ; you can redistribute it and/or modify it under the terms of the
 *   GNU General Public License as published by the Free Software Foundation ; either version 3 of
 *   the License, or (at your option) any later version.
 *
 *   TuringSim is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY ;
 *   without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along with TuringSim ;
 *   if not, write to the Free Software Foundation,
 *
 *   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 *
 *   Author : Ludovic Favre <ludovic.favre@epfl.ch>
 *
 *   Project supervisor : Mahdi Cheraghchi <mahdi.cheraghchi@epfl.ch>
 *
 *   Web site : http://icwww.epfl.ch/~lufavre
 *
 */

package gui;

import core.Alphabet;
import javax.swing.JPanel;

/**
 *
 * @author Ludovic Favre
 */
public class EditAlphabetDialog extends javax.swing.JDialog {

    private JPanel panel_;
    private Alphabet current_;
    private boolean isTapeAlphabet_;

    /** Creates new form EditAlphabetDialog */
    public EditAlphabetDialog(java.awt.Frame parent, boolean modal, JPanel panel, Alphabet current, boolean tape) {
        super(parent, modal);
        this.panel_=panel;
        this.current_ = current;
        this.isTapeAlphabet_ = tape;
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        alphabetTextField = new javax.swing.JTextField();
        alphabetLabel = new javax.swing.JLabel();
        hintLabel = new javax.swing.JLabel();
        validateButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(221, 150));
        setName("Form"); // NOI18N

        alphabetTextField.setText(this.current_.toString());
        alphabetTextField.setName("alphabetTextField"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(gui.TuringApp.class).getContext().getResourceMap(EditAlphabetDialog.class);
        alphabetLabel.setText(resourceMap.getString("alphabetLabel.text")); // NOI18N
        alphabetLabel.setName("alphabetLabel"); // NOI18N

        hintLabel.setFont(resourceMap.getFont("hintLabel.font")); // NOI18N
        hintLabel.setText(resourceMap.getString("hintLabel.text")); // NOI18N
        hintLabel.setName("hintLabel"); // NOI18N

        validateButton.setText(resourceMap.getString("validateButton.text")); // NOI18N
        validateButton.setName("validateButton"); // NOI18N
        validateButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                validateButtonMouseClicked(evt);
            }
        });

        cancelButton.setText(resourceMap.getString("cancelButton.text")); // NOI18N
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                cancelButtonMouseClicked(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(alphabetTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 180, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(hintLabel)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(validateButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cancelButton))
                    .addComponent(alphabetLabel))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(alphabetLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(alphabetTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(hintLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(validateButton)
                    .addComponent(cancelButton))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void validateButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_validateButtonMouseClicked
        if(isTapeAlphabet_){
            if(this.panel_ instanceof SingleTapePanel){
                ((SingleTapePanel)this.panel_).newTapeAlphabetIs(this.alphabetTextField.getText());
            }
            else{
                throw new IllegalArgumentException("Cannot use this class to change a multitape tape alphabet");
            }
            

        }
        else{
            if(this.panel_ instanceof SingleTapePanel){
                ((SingleTapePanel)this.panel_).newAlphabetIs(this.alphabetTextField.getText());
            }
            else if(this.panel_ instanceof MultiTapePanel){
                ((MultiTapePanel)this.panel_).newAlphabetIs(this.alphabetTextField.getText());
            }
            
        }

        this.dispose();
    }//GEN-LAST:event_validateButtonMouseClicked

    private void cancelButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_cancelButtonMouseClicked
        this.dispose();
    }//GEN-LAST:event_cancelButtonMouseClicked


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel alphabetLabel;
    private javax.swing.JTextField alphabetTextField;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel hintLabel;
    private javax.swing.JButton validateButton;
    // End of variables declaration//GEN-END:variables

}
