/*
 *   Copyright (©) 2009 | 16 January 2009 | EPFL (Ecole Polytechnique fédérale de Lausanne)
 *
 *   TuringSim is free software ; you can redistribute it and/or modify it under the terms of the
 *   GNU General Public License as published by the Free Software Foundation ; either version 3 of
 *   the License, or (at your option) any later version.
 *
 *   TuringSim is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY ;
 *   without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along with TuringSim ;
 *   if not, write to the Free Software Foundation,
 *
 *   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 *
 *   Author : Ludovic Favre <ludovic.favre@epfl.ch>
 *
 *   Project supervisor : Mahdi Cheraghchi <mahdi.cheraghchi@epfl.ch>
 *
 *   Web site : http://icwww.epfl.ch/~lufavre
 *
 */
package gui;

import core.Alphabet;
import java.util.HashSet;

/**
 *
 * @author Ludovic Favre
 */
public class EditMultiTapeTapeDialog extends javax.swing.JDialog {

    private HashSet<String> labels_;
    private MultiTapePanel panel_;
    private boolean inited_ = false;

    /** Creates new form EditMultiTapeTapeDialog */
    public EditMultiTapeTapeDialog(java.awt.Frame parent, boolean modal, MultiTapePanel panel, HashSet<String> labels) {
        super(parent, modal);
        
        this.panel_ = panel;
        this.labels_ = labels;
        this.inited_ = false;
        //System.out.println("Got labels : "+labels.toString());
        initComponents();
        initSelection();
        refreshContent();
        this.tapeComboBox.setSelectedIndex(0);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        hintLabel = new javax.swing.JLabel();
        validateButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        tapeContentTextField = new javax.swing.JTextField();
        tapeLabel = new javax.swing.JLabel();
        tapeComboBox = new javax.swing.JComboBox();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setName("Form"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(gui.TuringApp.class).getContext().getResourceMap(EditMultiTapeTapeDialog.class);
        hintLabel.setFont(resourceMap.getFont("hintLabel.font")); // NOI18N
        hintLabel.setText(resourceMap.getString("hintLabel.text")); // NOI18N
        hintLabel.setName("hintLabel"); // NOI18N

        validateButton.setText(resourceMap.getString("validateButton.text")); // NOI18N
        validateButton.setName("validateButton"); // NOI18N
        validateButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                validateButtonMouseClicked(evt);
            }
        });

        cancelButton.setText(resourceMap.getString("cancelButton.text")); // NOI18N
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                cancelButtonMouseClicked(evt);
            }
        });

        tapeContentTextField.setName("tapeContentTextField"); // NOI18N

        tapeLabel.setText(resourceMap.getString("tapeLabel.text")); // NOI18N
        tapeLabel.setName("tapeLabel"); // NOI18N

        tapeComboBox.setName("tapeComboBox"); // NOI18N
        tapeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[]{}));
        tapeComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tapeComboBoxActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(hintLabel)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(validateButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cancelButton))
                    .addComponent(tapeContentTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 180, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(tapeLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(tapeComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap(59, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(42, 42, 42)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(tapeLabel)
                    .addComponent(tapeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(10, 10, 10)
                .addComponent(tapeContentTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(hintLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(validateButton)
                    .addComponent(cancelButton))
                .addContainerGap(40, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
private void initSelection() {
        if (inited_) {
            this.tapeComboBox.removeAllItems();
            for (String string : labels_) {
                System.out.println("Item :"+string);
                this.tapeComboBox.addItem(string);
            }
        } else {
            inited_ = true;
            for (String string : labels_) {
                System.out.println("Item :"+string);
                this.tapeComboBox.addItem(string);
            }
        }
        System.out.println("Content size is now : "+this.tapeComboBox.getModel().getSize());
    }

    private void refreshContent() {
        String itemLabel = ((String) this.tapeComboBox.getModel().getSelectedItem()).toString();
        System.out.println("Selected item : "+itemLabel);
        if (this.panel_.tapeContent(itemLabel) != null) {
            System.out.println("Setting text to "+this.panel_.tapeContent(itemLabel));
            this.tapeContentTextField.setText(this.panel_.tapeContent(itemLabel));
            
        } else {
            System.out.println("FAILED TO RECOVER ITEM LABEL ! ");
        }

    }
    private void validateButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_validateButtonMouseClicked
        this.panel_.newTapeContentIs((String) this.tapeComboBox.getSelectedItem(), this.tapeContentTextField.getText());

        this.dispose();
}//GEN-LAST:event_validateButtonMouseClicked

    private void cancelButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_cancelButtonMouseClicked
        this.dispose();
}//GEN-LAST:event_cancelButtonMouseClicked

    private void tapeComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tapeComboBoxActionPerformed
        this.refreshContent();
}//GEN-LAST:event_tapeComboBoxActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel hintLabel;
    private javax.swing.JComboBox tapeComboBox;
    private javax.swing.JTextField tapeContentTextField;
    private javax.swing.JLabel tapeLabel;
    private javax.swing.JButton validateButton;
    // End of variables declaration//GEN-END:variables
}
