/*
 *   Copyright (©) 2009 | 16 January 2009 | EPFL (Ecole Polytechnique fédérale de Lausanne)
 *
 *   TuringSim is free software ; you can redistribute it and/or modify it under the terms of the
 *   GNU General Public License as published by the Free Software Foundation ; either version 3 of
 *   the License, or (at your option) any later version.
 *
 *   TuringSim is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY ;
 *   without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along with TuringSim ;
 *   if not, write to the Free Software Foundation,
 *
 *   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 *
 *   Author : Ludovic Favre <ludovic.favre@epfl.ch>
 *
 *   Project supervisor : Mahdi Cheraghchi <mahdi.cheraghchi@epfl.ch>
 *
 *   Web site : http://icwww.epfl.ch/~lufavre
 *
 */
/*
 * SingleTapePanel.java
 *
 */
package gui;

import core.Alphabet;
import core.SimpleTapeRunner;
import core.SimpleTuringMachine;
import core.Transition;
import core.exception.InvalidStateException;
import core.exception.InvalidTransitionException;
import core.exception.StateNotFoundException;
import core.exception.TapeBoundReachedException;
import core.io.XMLWriter;
import java.io.File;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import org.jdesktop.application.Action;

/**
 *
 * @author speedy
 */
public class SingleTapePanel extends javax.swing.JPanel implements TuringPanelInterface {

    /** Creates new form SingleTapePanel */
    public SingleTapePanel(SimpleTuringMachine tm, javax.swing.JTabbedPane tabReference, int intTabIndex, String filepath) {
        this.tm_ = tm;
        //System.out.println("Got my reference : " + inTabIndex + " and size is " + tabReference.getTabCount());
        //System.out.println("My path is :"+filepath);
        this.tabReference_ = tabReference;
        this.intTabIndex_ = intTabIndex;
        this.filepath_ = filepath;
        this.tm_.init();
        this.needSaving_ = false;
        try {
            this.runner_ = new SimpleTapeRunner(this.tm_);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this, "Turing Runner Engine initialization failed : " + e.getMessage());
        }

        initComponents();
        refreshAll();
        this.consoleAppend("Single Tape Turing Machine loaded : " + this.tm_.name());
        this.consoleAppend("Description : " + this.tm_.description());
        this.consoleAppend("-------------------------------------------------------");
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        transitionsTableScrollPane = new javax.swing.JScrollPane();
        transitionsTable = new javax.swing.JTable();
        statesListScrollPane = new javax.swing.JScrollPane();
        statesList = new javax.swing.JList();
        addStateButton = new javax.swing.JButton();
        deleteStateButton = new javax.swing.JButton();
        statesLabel = new javax.swing.JLabel();
        transitionsLabel = new javax.swing.JLabel();
        tapeLabel = new javax.swing.JLabel();
        addTransitionButton = new javax.swing.JButton();
        removeTransitionButton = new javax.swing.JButton();
        bottomSeparator = new javax.swing.JSeparator();
        tapeEditButton = new javax.swing.JButton();
        consoleLabel = new javax.swing.JLabel();
        consoleScrollPane = new javax.swing.JScrollPane();
        consoleTextPane = new javax.swing.JTextPane();
        closeTabButton = new javax.swing.JButton();
        controlsLabel = new javax.swing.JLabel();
        rewindButton = new javax.swing.JButton();
        stepButton = new javax.swing.JButton();
        tapeDisplayField = new javax.swing.JTextField();
        startingStateLabel = new javax.swing.JLabel();
        startingStateTextField = new javax.swing.JTextField();
        fullRunButton = new javax.swing.JButton();

        setName("Form"); // NOI18N

        transitionsTableScrollPane.setName("transitionsTableScrollPane"); // NOI18N

        this.model_ = new DefaultTableModel();
        this.model_.addColumn("State");
        this.model_.addColumn("Input");
        this.model_.addColumn("Next State");
        this.model_.addColumn("Output");
        this.model_.addColumn("Direction");
        transitionsTable.setModel(model_);
        transitionsTable.setName("transitionsTable"); // NOI18N
        transitionsTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                transitionsTableMouseClicked(evt);
            }
        });
        transitionsTableScrollPane.setViewportView(transitionsTable);

        statesListScrollPane.setName("statesListScrollPane"); // NOI18N

        statesList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        statesList.setName("statesList"); // NOI18N
        statesListScrollPane.setViewportView(statesList);

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(gui.TuringApp.class).getContext().getResourceMap(SingleTapePanel.class);
        addStateButton.setText(resourceMap.getString("addStateButton.text")); // NOI18N
        addStateButton.setName("addStateButton"); // NOI18N
        addStateButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                addStateButtonMouseClicked(evt);
            }
        });

        deleteStateButton.setText(resourceMap.getString("deleteStateButton.text")); // NOI18N
        deleteStateButton.setName("deleteStateButton"); // NOI18N
        deleteStateButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                deleteStateButtonMouseClicked(evt);
            }
        });

        statesLabel.setText(resourceMap.getString("statesLabel.text")); // NOI18N
        statesLabel.setName("statesLabel"); // NOI18N

        transitionsLabel.setText(resourceMap.getString("transitionsLabel.text")); // NOI18N
        transitionsLabel.setName("transitionsLabel"); // NOI18N

        tapeLabel.setText(resourceMap.getString("tapeLabel.text")); // NOI18N
        tapeLabel.setName("tapeLabel"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(gui.TuringApp.class).getContext().getActionMap(SingleTapePanel.class, this);
        addTransitionButton.setAction(actionMap.get("addTransition")); // NOI18N
        addTransitionButton.setText(resourceMap.getString("addTransitionButton.text")); // NOI18N
        addTransitionButton.setName("addTransitionButton"); // NOI18N
        addTransitionButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                addTransitionButtonMouseClicked(evt);
            }
        });

        removeTransitionButton.setAction(actionMap.get("removeTransition")); // NOI18N
        removeTransitionButton.setText(resourceMap.getString("removeTransitionButton.text")); // NOI18N
        removeTransitionButton.setName("removeTransitionButton"); // NOI18N
        removeTransitionButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                removeTransitionButtonMouseClicked(evt);
            }
        });

        bottomSeparator.setName("bottomSeparator"); // NOI18N

        tapeEditButton.setText(resourceMap.getString("tapeEditButton.text")); // NOI18N
        tapeEditButton.setName("tapeEditButton"); // NOI18N
        tapeEditButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                tapeEditButtonMouseClicked(evt);
            }
        });

        consoleLabel.setText(resourceMap.getString("consoleLabel.text")); // NOI18N
        consoleLabel.setName("consoleLabel"); // NOI18N

        consoleScrollPane.setName("consoleScrollPane"); // NOI18N

        consoleTextPane.setName("consoleTextPane"); // NOI18N
        consoleScrollPane.setViewportView(consoleTextPane);

        closeTabButton.setIcon(resourceMap.getIcon("closeTabButton.icon")); // NOI18N
        closeTabButton.setAlignmentX(1.0F);
        closeTabButton.setAlignmentY(1.0F);
        closeTabButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        closeTabButton.setMaximumSize(new java.awt.Dimension(20, 20));
        closeTabButton.setMinimumSize(new java.awt.Dimension(20, 20));
        closeTabButton.setName("closeTabButton"); // NOI18N
        closeTabButton.setPreferredSize(new java.awt.Dimension(20, 20));
        closeTabButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                closeTabButtonMouseClicked(evt);
            }
        });

        controlsLabel.setText(resourceMap.getString("controlsLabel.text")); // NOI18N
        controlsLabel.setName("controlsLabel"); // NOI18N

        rewindButton.setIcon(resourceMap.getIcon("rewindButton.icon")); // NOI18N
        rewindButton.setToolTipText(resourceMap.getString("rewindButton.toolTipText")); // NOI18N
        rewindButton.setActionCommand(resourceMap.getString("rewindButton.actionCommand")); // NOI18N
        rewindButton.setMaximumSize(new java.awt.Dimension(25, 25));
        rewindButton.setMinimumSize(new java.awt.Dimension(25, 25));
        rewindButton.setName("rewindButton"); // NOI18N
        rewindButton.setPreferredSize(new java.awt.Dimension(25, 25));
        rewindButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                rewindButtonMouseClicked(evt);
            }
        });

        stepButton.setIcon(resourceMap.getIcon("stepButton.icon")); // NOI18N
        stepButton.setActionCommand(resourceMap.getString("stepButton.actionCommand")); // NOI18N
        stepButton.setMaximumSize(new java.awt.Dimension(25, 25));
        stepButton.setMinimumSize(new java.awt.Dimension(25, 25));
        stepButton.setName("stepButton"); // NOI18N
        stepButton.setPreferredSize(new java.awt.Dimension(25, 25));
        stepButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                stepButtonMouseClicked(evt);
            }
        });

        tapeDisplayField.setBackground(resourceMap.getColor("tapeDisplayField.background")); // NOI18N
        tapeDisplayField.setEditable(false);
        tapeDisplayField.setFont(resourceMap.getFont("tapeDisplayField.font")); // NOI18N
        tapeDisplayField.setText(resourceMap.getString("tapeDisplayField.text")); // NOI18N
        tapeDisplayField.setName("tapeDisplayField"); // NOI18N

        startingStateLabel.setText(resourceMap.getString("startingStateLabel.text")); // NOI18N
        startingStateLabel.setName("startingStateLabel"); // NOI18N

        startingStateTextField.setEditable(false);
        startingStateTextField.setText(resourceMap.getString("startingStateTextField.text")); // NOI18N
        startingStateTextField.setEnabled(false);
        startingStateTextField.setName("startingStateTextField"); // NOI18N

        fullRunButton.setIcon(resourceMap.getIcon("fullRunButton.icon")); // NOI18N
        fullRunButton.setText(resourceMap.getString("fullRunButton.text")); // NOI18N
        fullRunButton.setMaximumSize(new java.awt.Dimension(25, 25));
        fullRunButton.setMinimumSize(new java.awt.Dimension(25, 25));
        fullRunButton.setName("fullRunButton"); // NOI18N
        fullRunButton.setPreferredSize(new java.awt.Dimension(25, 25));
        fullRunButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                fullRunButtonMouseClicked(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap(807, Short.MAX_VALUE)
                .addComponent(closeTabButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addGap(27, 27, 27)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(tapeLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(tapeDisplayField, javax.swing.GroupLayout.DEFAULT_SIZE, 739, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(statesListScrollPane, 0, 0, Short.MAX_VALUE)
                            .addComponent(addStateButton)
                            .addComponent(deleteStateButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(startingStateLabel)
                                .addGap(10, 10, 10)
                                .addComponent(startingStateTextField))
                            .addComponent(statesLabel))
                        .addGap(42, 42, 42)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(removeTransitionButton, javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(addTransitionButton, javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(transitionsLabel)
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(controlsLabel)
                                        .addGap(35, 35, 35)
                                        .addComponent(rewindButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(stepButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(fullRunButton, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 363, Short.MAX_VALUE)
                                        .addComponent(tapeEditButton))
                                    .addComponent(transitionsTableScrollPane, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 637, Short.MAX_VALUE))))))
                .addContainerGap())
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addGap(37, 37, 37)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(bottomSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 778, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(consoleLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(consoleScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 708, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(closeTabButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(46, 46, 46)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(tapeLabel)
                            .addComponent(tapeDisplayField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(17, 17, 17)
                        .addComponent(controlsLabel)
                        .addGap(18, 18, 18)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(statesLabel)
                            .addComponent(transitionsLabel)))
                    .addGroup(layout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(fullRunButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(stepButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(tapeEditButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(rewindButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 24, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(statesListScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 259, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(startingStateLabel)
                            .addComponent(startingStateTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(transitionsTableScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 282, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(addStateButton)
                    .addComponent(addTransitionButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(deleteStateButton)
                    .addComponent(removeTransitionButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 81, Short.MAX_VALUE)
                .addComponent(bottomSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(consoleLabel)
                        .addGap(94, 94, 94))
                    .addComponent(consoleScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 109, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void transitionsTableMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_transitionsTableMouseClicked
        if (evt.getButton() == java.awt.event.MouseEvent.BUTTON1) {
            System.out.println("Button 1");
        } else {
            if (evt.getButton() == java.awt.event.MouseEvent.BUTTON3 || evt.getButton() == java.awt.event.MouseEvent.BUTTON2) {
                System.out.println("Button 2");
            }
        }
}//GEN-LAST:event_transitionsTableMouseClicked

    private void addStateButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_addStateButtonMouseClicked
        addNewStateToList();
}//GEN-LAST:event_addStateButtonMouseClicked

    private void deleteStateButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_deleteStateButtonMouseClicked
        JFrame mainFrame = TuringApp.getApplication().getMainFrame();
        DeleteStateDialog addDialog = new DeleteStateDialog(mainFrame, this, true, this.tm_.states());
        addDialog.setLocationRelativeTo(mainFrame);
        TuringApp.getApplication().show(addDialog);
}//GEN-LAST:event_deleteStateButtonMouseClicked

    private void addTransitionButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_addTransitionButtonMouseClicked
        JFrame mainFrame = TuringApp.getApplication().getMainFrame();
        AddTransitionDialog addDialog = new AddTransitionDialog(mainFrame, true, this, this.tm_.states(), this.tm_.alphabet(), this.tm_.tapeAlphabet());
        addDialog.setLocationRelativeTo(mainFrame);
        TuringApp.getApplication().show(addDialog);
}//GEN-LAST:event_addTransitionButtonMouseClicked

    private void removeTransitionButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_removeTransitionButtonMouseClicked
        int row = this.transitionsTable.getSelectedRow();
        if (row != -1) {
            DefaultTableModel m = (DefaultTableModel) this.transitionsTable.getModel();
            /*Remainder : content of a row is an Object[]
             *
             * m1.addRow(new Object[]{transition.currentState(), transition.currentChar(),
            transition.nextState(),transition.nextChar(), transition.direction()});
             */
            core.State fromState = (core.State) m.getValueAt(row, 0);
            core.State toState = (core.State) m.getValueAt(row, 2);



            char inputSymb = ((Character) m.getValueAt(row, 1)).charValue();
            char ouputSymb = ((Character) m.getValueAt(row, 3)).charValue();

            core.Direction dir = (core.Direction) m.getValueAt(row, 4);

            Transition t = new Transition(fromState, toState, inputSymb, ouputSymb, dir);

            this.tm_.removeTransition(t);

            m.removeRow(row);

            refreshAll();
        }

}//GEN-LAST:event_removeTransitionButtonMouseClicked

    private void tapeEditButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tapeEditButtonMouseClicked
        JFrame mainFrame = TuringApp.getApplication().getMainFrame();
        EditTapeDialog tapeDialog = new EditTapeDialog(mainFrame, this, true, this.tm_.tapeDefaultContent());
        tapeDialog.setLocationRelativeTo(mainFrame);
        TuringApp.getApplication().show(tapeDialog);
}//GEN-LAST:event_tapeEditButtonMouseClicked

    private void closeTabButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_closeTabButtonMouseClicked
        if (this.needSaving_) {
            int ret = JOptionPane.showConfirmDialog(this, "Changes haven't been saved, do you really want to exit ?", "File not saved", WIDTH);
            if (ret == 0) {
                tabReference_.remove(this.intTabIndex_);
            }
        } else {
            tabReference_.remove(this.intTabIndex_);
        }

    }//GEN-LAST:event_closeTabButtonMouseClicked

    private void rewindButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_rewindButtonMouseClicked
        this.tm_.init();
        this.refreshAll();
        this.consoleAppend("Turing Machine has been resetted to default");
        this.consoleAppend("-------------------------------------------------------");
    }//GEN-LAST:event_rewindButtonMouseClicked

    private void stepButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_stepButtonMouseClicked
        try {
            boolean done = this.runner_.doStep();
            if (done) {
                this.consoleAppend("New current state : " + this.tm_.currentState().label());
            }
            this.refreshAll();
            if (!done) {
                if (this.tm_.accepting()) {
                    this.consoleAppend("No transition found, Turing machine state is " + this.tm_.currentState().label() + "(accepting)");
                }
                if (this.tm_.rejecting()) {
                    this.consoleAppend("No transition found, Turing machine state is " + this.tm_.currentState().label() + "(rejecting)");
                }
            }
        } catch (InvalidTransitionException e1) {
            JOptionPane.showMessageDialog(this, "Invalid transition exception");
        } catch (StateNotFoundException e2) {
            JOptionPane.showMessageDialog(this, "State not found exception");
        } catch (TapeBoundReachedException e3) {
            JOptionPane.showMessageDialog(this, "Tape bound reached !");
        }

    }//GEN-LAST:event_stepButtonMouseClicked

    private void fullRunButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_fullRunButtonMouseClicked
        boolean done = false;
        while (!done) {


            try {
                done = !this.runner_.doStep();
                if (!done) {
                    this.consoleAppend("New current state : " + this.tm_.currentState().label());
                }
                this.refreshAll();
                if (done) {
                    if (this.tm_.accepting()) {
                        this.consoleAppend("No transition found, Turing machine state is " + this.tm_.currentState().label() + "(accepting)");
                    }
                    if (this.tm_.rejecting()) {
                        this.consoleAppend("No transition found, Turing machine state is " + this.tm_.currentState().label() + "(rejecting)");
                    }
                }

            } catch (InvalidTransitionException e1) {
                JOptionPane.showMessageDialog(this, "Invalid transition exception");
            } catch (StateNotFoundException e2) {
                JOptionPane.showMessageDialog(this, "State not found exception");
            } catch (TapeBoundReachedException e3) {
                JOptionPane.showMessageDialog(this, "Tape bound reached !");
            }
        }

}//GEN-LAST:event_fullRunButtonMouseClicked

    public void addNewTransition(Transition t) {
        try {
            this.tm_.addTransition(t);
            this.refreshTransitions();
            this.needSaving_ = true;
        } catch (InvalidTransitionException e) {
            JOptionPane.showMessageDialog(this, "Failed to add the transition. Error is : \n" + e.getMessage());
        }
    }

    public void newTapeDefault(String newContent) {
        try {
            this.tm_.tapeContentIs(newContent);
            this.tm_.init();
            this.refreshAll();
            this.needSaving_ = true;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this, "Entered tape content contains invalid symbols. Tape content not changed ");
        }
    }

    private void refreshAll() {
        this.refreshStates();
        this.refreshTape();
        this.refreshTransitions();
    }

    private void refreshTape() {
        this.tapeDisplayField.setText(this.tm_.tapePositionString());

    }

    private void refreshStates() {
        this.statesList.removeAll();
        //System.out.println("Adding states ... !!!");
        Object[] obj = new Object[this.tm_.states().size()];
        for (int i = 0; i < obj.length; i++) {
            if (this.tm_.containsAcceptingState(this.tm_.states().get(i).toString())) {
                obj[i] = this.tm_.states().get(i).toString() + "<A>";
            } else if (this.tm_.containsRejectingState(this.tm_.states().get(i).toString())) {
                obj[i] = this.tm_.states().get(i).toString() + "<R>";
            } else {
                obj[i] = this.tm_.states().get(i).toString();
            }
        }

        final Object[] objs = obj;

        statesList.setModel(new javax.swing.AbstractListModel() {

            @Override
            public int getSize() {
                return objs.length;
            }

            @Override
            public Object getElementAt(int i) {
                return objs[i];
            }
        });

        this.startingStateTextField.setText(this.tm_.startState().toString());

    }

    private void refreshTransitions() {
        java.util.LinkedList<core.Transition> transitions = this.tm_.transitions();
        java.util.Collections.sort(transitions);
        TableModel model = this.transitionsTable.getModel();
        DefaultTableModel m1 = (DefaultTableModel) model;

        while (m1.getRowCount() != 0) {
            m1.removeRow(0);
        }

        for (Transition transition : transitions) {
            m1.addRow(new Object[]{transition.currentState(), transition.currentChar(),
                        transition.nextState(), transition.nextChar(), transition.direction()});
        }
    }

    @Action
    private void addNewStateToList() {
        JFrame mainFrame = TuringApp.getApplication().getMainFrame();
        AddStateDialog addDialog = new AddStateDialog(this, null, mainFrame, true);//AddStateDialog(this,mainFrame, true);
        addDialog.setLocationRelativeTo(mainFrame);
        TuringApp.getApplication().show(addDialog);
    }

    @Override
    public void addStateToList(core.State state, boolean starting, boolean accepting, boolean rejecting) {
        if ((accepting == rejecting) && accepting == true) {
            JOptionPane.showMessageDialog(this, "Couldn't insert the state as its options are not valid !");
        } else {

            try {
                this.tm_.addState(state);
                if (starting) {
                    this.tm_.startStateIs(state);
                }
                if (accepting) {
                    this.tm_.addAcceptingState(state.label());
                }
                if (rejecting) {
                    this.tm_.addRejectingState(state.label());
                }
            } catch (InvalidStateException e) {
                JOptionPane.showMessageDialog(this, "Cannot add state : " + state.label() + "\n" + e.getMessage());
                System.err.println(e.getMessage());
            } catch (Exception e) {
                this.tm_.removeState(state);
                JOptionPane.showMessageDialog(this, "Some settings of the state failed to be applied. \n" +
                        "The state has not been created");
                System.err.println(e.getMessage());
            }
        }

        this.refreshStates();
        this.needSaving_ = true;
    }

    @Override
    public void deleteState(core.State state) {
        this.tm_.removeState(state);
        this.refreshStates();
        this.needSaving_ = true;
    }

    @Override
    public void save() {
        if (this.filepath_ == null) {
            this.saveTo();
        } else {
            try {
                new XMLWriter().save(this.tm_, filepath_);
                this.needSaving_ = false;
            } catch (Exception e) {
                JOptionPane.showMessageDialog(this, "Failed to save the Turing Machine : \n" + e.getMessage());
            }
        }
    }

    @Override
    public void newAlphabetIs(String newAlphabet) {
        this.tm_.alphabetIs(new Alphabet(newAlphabet));
        this.needSaving_ = true;
    }

    public void newTapeAlphabetIs(String newTapeAlphabet) {
        this.tm_.tapeAlphabetIs(new Alphabet(newTapeAlphabet));
        this.needSaving_ = true;
    }

    public void editInputAlphabet() {
        JFrame mainFrame = TuringApp.getApplication().getMainFrame();
        EditAlphabetDialog editDialog = new EditAlphabetDialog(mainFrame, true, this, this.tm_.alphabet(), false);
        editDialog.setLocationRelativeTo(mainFrame);
        TuringApp.getApplication().show(editDialog);
    }

    public void editTapeAlphabet() {
        JFrame mainFrame = TuringApp.getApplication().getMainFrame();
        EditAlphabetDialog editDialog = new EditAlphabetDialog(mainFrame, true, this, this.tm_.tapeAlphabet(), true);
        editDialog.setLocationRelativeTo(mainFrame);
        TuringApp.getApplication().show(editDialog);
    }

    public void editTmInfo() {
        JFrame mainFrame = TuringApp.getApplication().getMainFrame();
        EditTuringMachineInformationDialog editDialog = new EditTuringMachineInformationDialog(mainFrame, true, this, this.tm_.name(), this.tm_.description());
        editDialog.setLocationRelativeTo(mainFrame);
        TuringApp.getApplication().show(editDialog);
    }

    public void tmInfoAre(String name, String description) {
        this.tm_.nameIs(name);
        this.tm_.descriptionIs(description);
        this.needSaving_ = true;
    }

    @Override
    public void saveTo() {
        JFrame mainFrame = TuringApp.getApplication().getMainFrame();
        OpenFileBox openBox = new OpenFileBox(mainFrame, true,"Save as...");
        openBox.setLocationRelativeTo(mainFrame);


        TuringApp.getApplication().show(openBox);

        File selectedFile = openBox.jFileChooser().getSelectedFile();
        if (selectedFile == null) {
        } else {
            try {
                new XMLWriter().save(this.tm_, selectedFile.getAbsolutePath());
                this.needSaving_ = false;
                this.filepath_ = selectedFile.getAbsolutePath();
            } catch (Exception e) {
                JOptionPane.showMessageDialog(this, "Failed to save the Turing Machine : \n" + e.getMessage());
            }

        }
    }

    @Override
    public void consoleAppend(String text) {
        this.consoleTextPane.setText(this.consoleTextPane.getText() + "\n" + text);
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addStateButton;
    private javax.swing.JButton addTransitionButton;
    private javax.swing.JSeparator bottomSeparator;
    private javax.swing.JButton closeTabButton;
    private javax.swing.JLabel consoleLabel;
    private javax.swing.JScrollPane consoleScrollPane;
    private javax.swing.JTextPane consoleTextPane;
    private javax.swing.JLabel controlsLabel;
    private javax.swing.JButton deleteStateButton;
    private javax.swing.JButton fullRunButton;
    private javax.swing.JButton removeTransitionButton;
    private javax.swing.JButton rewindButton;
    private javax.swing.JLabel startingStateLabel;
    private javax.swing.JTextField startingStateTextField;
    private javax.swing.JLabel statesLabel;
    private javax.swing.JList statesList;
    private javax.swing.JScrollPane statesListScrollPane;
    private javax.swing.JButton stepButton;
    private javax.swing.JTextField tapeDisplayField;
    private javax.swing.JButton tapeEditButton;
    private javax.swing.JLabel tapeLabel;
    private javax.swing.JLabel transitionsLabel;
    private javax.swing.JTable transitionsTable;
    private javax.swing.JScrollPane transitionsTableScrollPane;
    // End of variables declaration//GEN-END:variables
    private DefaultTableModel model_ = new DefaultTableModel();
    private SimpleTuringMachine tm_;
    private boolean needSaving_ = false;
    private javax.swing.JTabbedPane tabReference_ = null;
    private int intTabIndex_ = -1;
    private String filepath_;
    private SimpleTapeRunner runner_;
}
